@@ -257,7 +257,8 @@ class GRAPH_MT_key(Menu):
         layout.operator_menu_enum("graph.easing_type", "type", text="Easing Type")
 
         layout.separator()
-        layout.operator("graph.clean")
+        layout.operator("graph.clean").channels = False
+        layout.operator("graph.clean", text="Clean Channels").channels = True
         layout.operator("graph.smooth")
         layout.operator("graph.sample")
         layout.operator("graph.bake")
@@ -292,7 +293,8 @@ class GRAPH_MT_delete(Menu):
 
         layout.separator()
 
-        layout.operator("graph.clean")
+        layout.operator("graph.clean").channels = False
+        layout.operator("graph.clean", text="Clean Channels").channels = True
 
 
 if __name__ == "__main__":  # only for live edit.
